package gui;

import gui.listener.TestConnectivityListener;
import gui.listener.UploadListener;

import java.awt.Dimension;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Properties;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.SpinnerDateModel;

import net.sourceforge.jdatepicker.JDateComponentFactory;
import net.sourceforge.jdatepicker.JDatePicker;

import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;


//	http://docs.oracle.com/javase/tutorial/uiswing/layout/box.html

public class ReportsGUIFrame extends JFrame {

	private static final int DEFAULT_WIDTH = 550;
	private static final int DEFAULT_HEIGHT = 350;
//	private static final String newline = System.getProperty("line.separator");
	private static final String REPORTS_PROPERTIES_FILE = "reports.properties";
	private static String REPORTS_SERVER_NAME = "reports.server.name";
	private static String REPORTS_SERVER_NAME_DEFAULT = "localhost";
	private static String REPORTS_SERVER_PORT = "reports.server.port";
	private static String REPORTS_SERVER_PORT_DEFAULT = "8080";
	private static final String REPORTS_ENDPOINT_PATH = "/servreports/services/reports";

	private String endpointUrl = "http://localhost:8080/servreports/services/reports";
	Properties properties = new Properties();

	JLabel testOutputLabel = new JLabel("?",JLabel.CENTER);
	JLabel uploadLabel = new JLabel(" ",JLabel.CENTER);
	
	Logger logger = Logger.getLogger(ReportsGUIFrame.class);

//	private SwingWorker<StringBuilder, ProgressData> textReader;
	
	public ReportsGUIFrame() {
	    
	    readProperties();
		
		Dimension horizSpace = new Dimension(10,0);
		Dimension vertSpace = new Dimension(0,20);
		
		JTabbedPane tabPane = new JTabbedPane(  );
	    
	    JPanel voltagePanel = new JPanel(  );
		BoxLayout voltagePanelMainBox = new BoxLayout(voltagePanel, BoxLayout.Y_AXIS);
		voltagePanel.setLayout(voltagePanelMainBox);
	    voltagePanel.add(new JLabel("Voltage Report Search Parameters"));

	    JPanel startTimePanel = new JPanel(  );
		BoxLayout startTimeBox = new BoxLayout(startTimePanel, BoxLayout.X_AXIS);
		startTimePanel.setLayout(startTimeBox);

		JLabel startTimeLabel = new JLabel("Start Date & Time");
		startTimePanel.add(startTimeLabel);
		
	    JDatePicker picker = JDateComponentFactory.createJDatePicker();
	    ((JComponent)picker).setAlignmentX(LEFT_ALIGNMENT);
	    Dimension prefSize = ((JComponent)picker).getPreferredSize();
	    ((JComponent)picker).setMaximumSize(prefSize);
	    startTimePanel.add(Box.createRigidArea(horizSpace));
	    startTimePanel.add((JComponent)picker);
	    
	    SpinnerDateModel sm = new SpinnerDateModel(new java.util.Date(), null, null,Calendar.HOUR_OF_DAY);
	    JSpinner timePicker = new JSpinner(sm);
	    Dimension dim = timePicker.getPreferredSize();
	    dim.height = prefSize.height;
	    timePicker.setMaximumSize(dim);
	    JSpinner.DateEditor de = new JSpinner.DateEditor(timePicker, "hh:mm");
	    timePicker.setEditor(de); 
	    timePicker.setAlignmentX(LEFT_ALIGNMENT);
	    startTimePanel.add(Box.createRigidArea(horizSpace));
	    startTimePanel.add(timePicker);
	    startTimePanel.add(Box.createHorizontalGlue());

	    JPanel endTimePanel = new JPanel(  );
		BoxLayout endTimeBox = new BoxLayout(endTimePanel, BoxLayout.X_AXIS);
		endTimePanel.setLayout(endTimeBox);

		JLabel endTimeLabel = new JLabel("End Date & Time  ");
		endTimePanel.add(endTimeLabel);
		
	    JDatePicker endPicker = JDateComponentFactory.createJDatePicker();
	    ((JComponent)endPicker).setAlignmentX(LEFT_ALIGNMENT);
	    ((JComponent)endPicker).setMaximumSize(prefSize);
	    endTimePanel.add(Box.createRigidArea(horizSpace));
	    endTimePanel.add((JComponent)endPicker);
	    
	    JSpinner endTimePicker = new JSpinner(sm);
	    endTimePicker.setMaximumSize(dim);
	    JSpinner.DateEditor endDateEditor = new JSpinner.DateEditor(endTimePicker, "hh:mm");
	    endTimePicker.setEditor(endDateEditor); 
	    endTimePicker.setAlignmentX(LEFT_ALIGNMENT);
	    endTimePanel.add(Box.createRigidArea(horizSpace));
	    endTimePanel.add(endTimePicker);
	    endTimePanel.add(Box.createHorizontalGlue());

	    
	    voltagePanel.add(Box.createRigidArea(vertSpace));
	    voltagePanel.add(startTimePanel);
	    voltagePanel.add(Box.createRigidArea(vertSpace));
	    voltagePanel.add(endTimePanel);
	    voltagePanel.add(Box.createVerticalGlue());

	    tabPane.addTab("Voltage Reports", voltagePanel);


	    JPanel messagePanel = new JPanel(  );
		BoxLayout messagePanelMainBox = new BoxLayout(messagePanel, BoxLayout.Y_AXIS);
		messagePanel.setLayout(messagePanelMainBox);
	    messagePanel.add(new JLabel("Message Success Rate Search Parameters"));

	    JPanel testPanel = new JPanel(  );
		BoxLayout testBox = new BoxLayout(testPanel, BoxLayout.X_AXIS);
		testPanel.setLayout(testBox);

		testPanel.add(Box.createHorizontalGlue());

		JButton testButton = new JButton("Test Connectivity");
		testPanel.add(testButton);
		testButton.addActionListener( new TestConnectivityListener(testOutputLabel, endpointUrl) );
		
	    Dimension d = testOutputLabel.getPreferredSize();  
	    testOutputLabel.setPreferredSize(new Dimension(d.width+300,d.height));

		testPanel.add(Box.createRigidArea(horizSpace));
		testPanel.add(testOutputLabel);
		testPanel.add(Box.createHorizontalGlue());

		messagePanel.add(Box.createRigidArea(vertSpace));
	    messagePanel.add(testPanel);
	    messagePanel.add(Box.createVerticalGlue());

	    //	Upload File
	    JPanel uploadPanel = new JPanel(  );
		BoxLayout uploadBox = new BoxLayout(uploadPanel, BoxLayout.X_AXIS);
		uploadPanel.setLayout(uploadBox);

		uploadPanel.add(Box.createHorizontalGlue());

		
		JButton selectFileButton = new JButton("Upload a File");
		uploadPanel.add(selectFileButton);
		selectFileButton.addActionListener( new UploadListener(this) );
		
	    d = uploadLabel.getPreferredSize();  
	    uploadLabel.setPreferredSize(new Dimension(d.width+300,d.height));

		uploadPanel.add(Box.createRigidArea(horizSpace));
		uploadPanel.add(uploadLabel);
		uploadPanel.add(Box.createHorizontalGlue());

		messagePanel.add(Box.createRigidArea(vertSpace));
	    messagePanel.add(uploadPanel);
	    messagePanel.add(Box.createVerticalGlue());

	    tabPane.addTab("Message Success Rate", messagePanel);
	    
	    add(tabPane);
	    
	    setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);
	}

	private void readProperties() {

		try {
		    properties.load(new FileInputStream(REPORTS_PROPERTIES_FILE));
		} catch (IOException e) {
		
			logger.error(e);
		}
	    
	    String serverName;
	    String s = (String)properties.get(REPORTS_SERVER_NAME);
	    if (StringUtils.isNotBlank(s))
	    	serverName = s;
	    else
	    	serverName = REPORTS_SERVER_NAME_DEFAULT;
	    
	    String port;
	    s = (String)properties.get(REPORTS_SERVER_PORT);
	    if (StringUtils.isNotBlank(s))
	    	port = s;
	    else 
	    	port = REPORTS_SERVER_PORT_DEFAULT;

	    endpointUrl = "http://" + serverName + ":" + port + REPORTS_ENDPOINT_PATH;
	    
	    logger.info("Endpoint URL : " + endpointUrl);

	}


	public JLabel getUploadLabel() {return uploadLabel;}
	
}
